////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Dclaration et dfinitions des priphriques              ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
#define _CS_MMC   0x43 // C
#define _CS_SerialFlash 0x53 // S
#define _CS_MP3   0x4D // M

////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              TFT module connections                                    ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
char TFT_DataPort at LATE;
sbit TFT_RST at LATC1_bit;
sbit TFT_BLED at LATA9_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATF12_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_BLED_Direction at TRISA9_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISF12_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
////////////////////////////////////////////////////////////////////////////////
////  Macros: PMPWaitBusy()                                                 ////
////                                                                        ////
////  Overview: Delays execution for PMP cycle time                         ////
////                                                                        ////
////                                                                        ////
#define PMPWaitBusy()   while(PMMODEbits.BUSY);

 void Set_Index(unsigned short index)
{
  TFT_RS = 0;
  PMDIN = index;
  PMPWaitBusy();
}

void Write_Command( unsigned short cmd )
{
  TFT_RS = 1;
  PMDIN = cmd;
  PMPWaitBusy();
}

void Write_Data(unsigned int _data)
{
  TFT_RS = 1;
  PMDIN = _data;
  PMPWaitBusy();
}

void Set_Address(unsigned x, unsigned y)
{
   Set_Index(0x02);
   Write_Command(hi(x));
   Set_Index(0x03);
   Write_Command(lo(x));
   Set_Index(0x06);
   Write_Command(hi(y));
   Set_Index(0x07);
   Write_Command(lo(y));
   Set_Index(0x22);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////  Function: ReadData()                                                  ////
////                                                                        ////
////  Overview: Read data                                                   ////
////  Input   : none                                                        ////
////  Output  : 16 bits data                                                ////
////                                                                        ////
unsigned ReadData(void)
{
   unsigned dat;

   TFT_RS = 1;
   dat = PMDIN;
   PMPWaitBusy();
   dat = PMDIN;
   return (dat);
}
////////////////////////////////////////////////////////////////////////////////
////  Function:  unsigned GetPixel(unsigned x, unsigned y)                  ////
////                                                                        ////
////  Overview: Gets color pixel at x,y                                     ////
////  Input:  x,y   - pixel coordinates                                     ////
////  Output: color - 16 bits color pixel in RGB565 format (65K)            ////
////  See HX8347-D Datasheet for more details                               ////
////                                                                        ////
unsigned GetPixel(unsigned x, unsigned y)
{
   unsigned val;
   char r,g,b;
   char out[15];
   
   TFT_CS = 0;
   Set_Address(x,y);

   ReadData();  // Dummy for initiate PMP reading
   ReadData();  // Dummy for initiate TFT reading
   val = ReadData();
   r = (char)hi(val)>>2;
   g = (char)lo(val)>>2;
   val = ReadData();
   b = (char)hi(val)>>2;
   TFT_CS = 1;
   
   wordToStr(r, out);
TFT_Write_Text(out, 160, 150);
   wordToStr(g, out);
TFT_Write_Text(out, 160, 160);
   wordToStr(b, out);
TFT_Write_Text(out, 160, 170);
   wordToStr(TFT_RGBToColor16bit(r, g, b), out);
TFT_Write_Text(out, 160, 180);
   return  TFT_RGBToColor16bit(r, g, b);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Initialisation du MCU                                     ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Init_MCU() {
    CM1CON  = 0;                    // Turn off comparators
    CM2CON  = 0;
    AD1PCFG = 0xff;                 // Make all inputs digital
    PMMODE = 0;
    PMAEN  = 0;
    PMCON  = 0;  // WRSP: Write Strobe Polarity bit
    PMMODEbits.MODE = 2;     // Master 2
    PMMODEbits.WAITB = 0;
    PMMODEbits.WAITM = 1;
    PMMODEbits.WAITE = 0;
    PMMODEbits.MODE16 = 1;   // 16 bit mode
    PMCONbits.CSF = 0;
    PMCONbits.PTRDEN = 1;
    PMCONbits.PTWREN = 1;
    PMCONbits.PMPEN = 1;
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              ADC initialisation                                        ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Init_ADC() {
   AD1PCFG = 0xFFFF;
   PCFG12_bit = 0;
   PCFG13_bit = 0;
   ADC1_Init();
}

////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              MMC Chip Select connection                                ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////                                                                                                                                                                        // Extern memory interface
sbit Mmc_Chip_Select at LATG9_bit;
sbit Mmc_Chip_Select_Direction at TRISG9_bit;
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Ampli Audio ShutDown                                      ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
sbit ST_SHUTDOWN at LATE8_bit;
sbit ST_SHUTDOWN_DIRECTION at TRISE8_bit;
sbit MO_SHUTDOWN at LATE9_bit;
sbit MO_SHUTDOWN_DIRECTION at TRISE9_bit;
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              CODEC MP3 Chip Select connection                          ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
sbit MP3_CS_Direction          at TRISG15_bit;
sbit MP3_CS                    at LATG15_bit;
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Touch Panel globals variables                             ////
////              Touch Pannel module connections                           ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 1000;
char PenDown;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

sbit DriveX_Left at LATB13_bit;
sbit DriveX_Right at LATB11_bit;
sbit DriveY_Up at LATB12_bit;
sbit DriveY_Down at LATB10_bit;
sbit DriveX_Left_Direction at TRISB13_bit;
sbit DriveX_Right_Direction at TRISB11_bit;
sbit DriveY_Up_Direction at TRISB12_bit;
sbit DriveY_Down_Direction at TRISB10_bit;
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////   Function : InitPMP(void)                                             ////
////   Overview : Initialize Paralel Bus between PIC and TFT screen         ////
////                                                                        ////
void Init_PMP(void) {
    PMMODE            = 0;
    PMAEN             = 0;
    PMCON             = 0;
    PMMODEbits.MODE   = 2;  // Master 2
    PMMODEbits.WAITB  = 0;
    PMMODEbits.WAITM  = 3;
    PMMODEbits.WAITE  = 0;
    PMMODEbits.MODE16 = 1;  // 16 bit mode
    PMCONbits.CSF     = 0;
    PMCONbits.PTRDEN  = 1;
    PMCONbits.PTWREN  = 1;
    PMCONbits.PMPEN   = 1;
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////           Initialisation du timer 1 - 50s                             ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////

void InitTimer1(){
  T1CON         = 0x8020;
  T1IE_bit      = 1;
  T1IF_bit      = 0;
  T1IP0_bit     = 1;
  T1IP1_bit     = 1;
  T1IP2_bit     = 1;
  PR1           = 62500;
  TMR1          = 0;
}
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Stereo ShutDown(char mode)                                ////
////              mode : ON - OFF                                           ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Stereo_ShutDown(char mode)
{
   ST_SHUTDOWN_DIRECTION = 0;
   if (mode == 1)   ST_SHUTDOWN = 0;
   else ST_SHUTDOWN = 1;
}
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Mono ShutDown(char mode)                                ////
////              mode : ON - OFF                                           ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Mono_ShutDown(char mode)
{
   MO_SHUTDOWN_DIRECTION = 0;
   MO_SHUTDOWN = mode;
}
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////           Initialisation de la dalle tactile                           ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
static void Init_TP() {
  Init_ADC();
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init(320, 240);
  TP_TFT_Init(320, 240, 13, 12);                   // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);         // Set touch panel ADC threshold
  TP_TFT_Set_Calibration_Consts(76,907,77,915);    // Set default calibration constants
  PenDown = 0;
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////           Calibration de la dalle tactile                              ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Calib_TP() {
  TFT_Set_Pen(CL_WHITE, 3);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Write_Text("Touch selected corners for calibration", 1, 150);
  TFT_Line(229, 309, 239, 319);
  TFT_Line(233, 319, 239, 319);
  TFT_Line(239, 313, 239, 319);
  TFT_Write_Text("first here", 160, 290);

  TP_TFT_Calibrate_Min();                      // Calibration of bottom left corner
  Delay_ms(500);

  TFT_Set_Pen(CL_BLACK, 3);
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Line(229, 309, 239, 319);
  TFT_Line(233, 319, 239, 319);
  TFT_Line(239, 313, 239, 319);
  TFT_Write_Text("first here", 160, 290);

  TFT_Set_Pen(CL_WHITE, 3);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Line(0, 0, 10, 10);
  TFT_Line(0, 0, 6, 0);
  TFT_Line(0, 0, 0, 6);
  TFT_Write_Text("now here", 20, 5);

  TP_TFT_Calibrate_Max();                      // Calibration of bottom left corner
  Delay_ms(500);
}

////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Initialisation global du port SPI                         ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
 void SPI_Device_CS_Enable_Disable(char Device_Name)
{
    switch(Device_Name)
    {
        case _CS_MMC:
        {
           CS_Serial_Flash_bit=1;
           Mp3_CS=1;
           Mmc_Chip_Select=0;
           CS_Serial_Flash_Direction_bit=0;
           Mmc_Chip_Select_Direction=0;
           MP3_CS_Direction=0;
           SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 4, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
           break;
        }
        case _CS_SerialFlash:
        {
           Mmc_Chip_Select_Direction=0;
           Mmc_Chip_Select=1;
           MP3_CS_Direction=0;
           Mp3_CS=1;
           CS_Serial_Flash_Direction_bit=0;
           CS_Serial_Flash_bit=0;

           SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 2, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_ACTIVE_2_IDLE);
           break;
        }
        case _CS_MP3:
        {
           Mmc_Chip_Select=1;
           CS_Serial_Flash_bit=1;
           Mp3_CS=0;
           CS_Serial_Flash_Direction_bit=0;
           Mmc_Chip_Select_Direction=0;
           MP3_CS_Direction=0;
           break;
        }
    }
    Delay_ms(500);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Initialisation de la Serial Flash                         ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void SF_Start(void)
{
   // Disable other peripheral modules on the same SPI bus
   Mmc_Chip_Select_Direction = 0;
   Mmc_Chip_Select = 1;            // Disable MMC module

   MP3_CS_Direction = 0;
   MP3_CS = 1;                     // Disable MP3 module

   // Initialize SPI2 module
   // master_mode   = _SPI_MASTER
   // data_mode     = _SPI_8_BIT
   // clock_divider = 1-1024
   // slave_select  = _SPI_SS_DISABLE (Only for slave mod)
   // data_sample   = _SPI_DATA_SAMPLE_MIDDLE
   // clock_idle    = _SPI_CLK_IDLE_LOW
   // edge          = _SPI_ACTIVE_2_IDLE
   SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, 8, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_ACTIVE_2_IDLE);
   SerialFlash_init();
   SerialFlash_WriteEnable();
   Delay_ms(500);
}