////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Si4703 Library                                            ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
sbit Si4703_RST at LATB14_bit;
sbit Si4703_RST_Direction at TRISB14_bit;
sbit Si4703_SDIO at LATA15_bit;
sbit Si4703_SDIO_Direction at TRISA15_bit;
sbit Si4703_SCLK at LATA14_bit;
sbit Si4703_SCLK_Direction at TRISA14_bit;

////                                                                        ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////             Prcurseur de fonction                                     ////
////                                                                        ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void      Si4703_Start();
void      Si4703_PowerON();
void      Si4703_Init();
void      Si4703_ReadRegisters();
void      Si4703_UpdateRegisters();
void      Si4703_SetChannel(unsigned channel);
char      Si4703_IsStereo();
char      Si4703_SeekUp();
char      Si4703_SeekDown();
void      Si4703_SetVolume(char vol);
void      Si4703_ReadRDS(rds *trame);
char      Si4703_RDSRdy(rds *trame);
char      Si4703_IsRDS();
int       Si4703_Seek(short Direction);
unsigned  Si4703_GetChannel();

////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////              Dclaration des variables                                 ////

unsigned si4703_registers[16];

static const unsigned  FAIL = 0;
static const unsigned  SUCCESS = 1;

static const char      SI4703 = 0x10;
static const unsigned  I2C_FAIL_MAX = 10;
static const unsigned  SEEK_DOWN = 0;
static const unsigned  SEEK_UP = 1;

//Define the register names
static const unsigned  DEVICEID = 0x00;
static const unsigned  CHIPID = 0x01;
static const unsigned  POWERCFG = 0x02;
static const unsigned  CHANNEL = 0x03;
static const unsigned  SYSCONFIG1 = 0x04;
static const unsigned  SYSCONFIG2 = 0x05;
static const unsigned  SYSCONFIG3 = 0x06;
static const unsigned  STATUSRSSI = 0x0A;
static const unsigned  READCHAN = 0x0B;
static const unsigned  RDSA = 0x0C;
static const unsigned  RDSB = 0x0D;
static const unsigned  RDSC = 0x0E;
static const unsigned  RDSD = 0x0F;

//Register 0x02 - POWERCFG
static const unsigned  SMUTE  = 15;
static const unsigned  DMUTE  = 14;
static const unsigned  MONO   = 13;
static const unsigned  RDSM   = 11;
static const unsigned  SKMODE = 10;
static const unsigned  SEEKUP =  9;
static const unsigned  SEEK   =  8;

//Register 0x03 - CHANNEL
static const unsigned  TUNE = 15;

//Register 0x04 - SYSCONFIG1
static const unsigned  RDSIEN = 15;
static const unsigned  RDS    = 12;
static const unsigned  DE     = 11;
static const unsigned  GPIO21 =  3;
static const unsigned  GPIO20 =  2;

//Register 0x05 - SYSCONFIG2
static const unsigned  SPACE1 =  5;
static const unsigned  SPACE0 =  4;

//Register 0x06 - SYSCONFIG3
static const unsigned  VOLEXT = 8;

//Register 0x0A - STATUSRSSI
static const unsigned  RDSR   = 15;
static const unsigned  STC    = 14;
static const unsigned  SFBL   = 13;
static const unsigned  AFCRL  = 12;
static const unsigned  RDSS   = 11;
static const unsigned  BLERA1 = 10;
static const unsigned  BLERA0 = 9;
static const unsigned  STEREO = 8;

//Register 0x0B - READCHAN
static const unsigned  BLERB1 = 15;
static const unsigned  BLERB0 = 14;
static const unsigned  BLERC1 = 13;
static const unsigned  BLERC0 = 12;
static const unsigned  BLERD1 = 11;
static const unsigned  BLERD0 = 10;


////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Allumage du module                                        ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_PowerON()
{
  Si4703_RST_DIRECTION = 0;
  Si4703_RST  = 1;
  delay_us(10);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Initialisation                                            ////
////              Mode I2C                                                  ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_Init()
{

  Si4703_ReadRegisters();
  si4703_registers[0x07] = 0x8100;
  Si4703_UpdateRegisters();
  delay_ms(500);
  
  Si4703_ReadRegisters();
  si4703_registers[POWERCFG]    = 0x0001;
  si4703_registers[POWERCFG]   |= (1<<RDSM);
  si4703_registers[SYSCONFIG1] |= (1<<RDS);
  si4703_registers[SYSCONFIG1] |= (1<<DE);

  si4703_registers[SYSCONFIG2] |= (1<<SPACE0);
  si4703_registers[SYSCONFIG2] &= 0xFFF0;
  si4703_registers[SYSCONFIG2] |= 0x0000;
  Si4703_UpdateRegisters();
  delay_ms(250);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Mise en veille du module                                  ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_Sleep()
{
  Si4703_ReadRegisters();
  si4703_registers[SYSCONFIG1] &= 0xCFFF;
  si4703_registers[POWERCFG] = 0x0005;
  Si4703_UpdateRegisters();
  delay_ms(250);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Reveil du module                                          ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_WakeUp(char vol, unsigned freq)
{
  Si4703_ReadRegisters();
  si4703_registers[POWERCFG]    = 0x4001;
  si4703_registers[POWERCFG]   |= (1<<RDSM);
  si4703_registers[SYSCONFIG1] |= (1<<RDS);
  Si4703_UpdateRegisters();

  Si4703_SetChannel(freq);
  
  if(vol < 0) vol = 0;
  if (vol > 31) vol = 31;
  if (vol < 16) si4703_registers[SYSCONFIG3] |= (1<<VOLEXT);
  else si4703_registers[SYSCONFIG3] &= 0xFEFF;

  si4703_registers[SYSCONFIG2] &= 0xFFF0;
  si4703_registers[SYSCONFIG2] |= vol;
  Si4703_UpdateRegisters();
  delay_ms(250);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Mise en service Tunner FM                                 ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_Start()
{
  I2C1_Start();
  Si4703_PowerON();
  TFT_Write_Text("Tuner FM : ON...", 10, 40);
  TFT_Write_Text("Boot...", 110, 40);
  Si4703_Init();
  TFT_Write_Text("En service...", 155, 40);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////              Lit tous les registres                                    ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_ReadRegisters()
{
  char i;
  char j;

  I2C1_Start();
  I2C1_Write(0x21);
  j = 0x0A;
  for(i = 0; i < 15 ; i++)
  {
    if (j == 0x10) j = 0;
    Si4703_registers[j] = I2C1_Read(_I2C_ACK) << 8;
    Si4703_registers[j] |= I2C1_Read(_I2C_ACK);
    j++;
  }
  Si4703_registers[j] = I2C1_Read(_I2C_ACK) << 8;
  Si4703_registers[j] |= I2C1_Read(_I2C_NACK);
  I2C1_Stop() ;
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                Met  jour les 6 registres principaux                   ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_UpdateRegisters()
{
  char i;
  char j;
  short high_byte;
  short low_byte;

  j = 0x02;
  I2C1_Start();
  I2C1_Write(0x20);
  for(i = 0; i < 7 ; i++)
  {
    high_byte = si4703_registers[j] >> 8;
    low_byte  = si4703_registers[j] & 0x00FF;
    I2C1_Write(high_byte);
    I2C1_Write(low_byte);
    j++;
  }
  I2C1_Stop();
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                Recherche la frquence suivante (Haut ou Bas)           ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
int Si4703_Seek(short Direction)
{
  int ValueSFBL;
  
  Si4703_ReadRegisters();
  si4703_registers[POWERCFG] |= (1<<SKMODE);
  if(Direction == SEEK_DOWN) si4703_registers[POWERCFG] &= ~(1<<SEEKUP);
  else si4703_registers[POWERCFG] |= 1<<SEEKUP;
  si4703_registers[POWERCFG] |= (1<<SEEK);
  Si4703_UpdateRegisters();
  while(1) {
    Si4703_ReadRegisters();
    if((si4703_registers[STATUSRSSI] & (1<<STC)) != 0) break;
  }
  Si4703_ReadRegisters();
  valueSFBL = si4703_registers[STATUSRSSI] & (1<<SFBL);
  si4703_registers[POWERCFG] &= ~(1<<SEEK);
  Si4703_updateRegisters();

  while(1) {
    Si4703_ReadRegisters();
    if( (si4703_registers[STATUSRSSI] & (1<<STC)) == 0) break;
  }
  if(valueSFBL) return(0);
  return Si4703_GetChannel();
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                Retour la frquence en cours                            ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
unsigned Si4703_GetChannel()
{
  int frequence;
  Si4703_ReadRegisters();
  frequence = si4703_registers[READCHAN] & 0x03FF;
  frequence += 875;
  return(frequence);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                indicateur STEREO/MONO                                  ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
char Si4703_IsStereo()
{
   char flag;
   
   si4703_ReadRegisters();
   if (si4703_registers[STATUSRSSI] & (1<<STEREO)) flag = 1;
   else flag = 0;
   return(flag);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                Force mode STEREO/MONO                                  ////
////                mode = 1 -> force MONO                                  ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_Stereo(short mode)
{
   si4703_ReadRegisters();
   if (mode == 1) si4703_registers[POWERCFG] |= (1<<MONO);
   else si4703_registers[POWERCFG] &= ~(1<<MONO);
   si4703_UpdateRegisters();
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                Lit la puissance du signal dtect                      ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
unsigned short Si4703_GetRSSI()
{
   unsigned short puiss,level;
   
   si4703_ReadRegisters();
   return (Lo(si4703_registers[STATUSRSSI]) / 10);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                fixe la sensibilit                                     ////
////       0 : plus sensib - 3 moins sensib                                 ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_SetSensib(short sensib)
{
  Si4703_ReadRegisters();
  si4703_registers[SYSCONFIG2] &= (0x00FF);
  si4703_registers[SYSCONFIG3] &= (0xFF00);
  switch (sensib)
  {
     case 0 :
       break;
     case 1 :
       si4703_registers[SYSCONFIG2] |= (0x0C00);
       si4703_registers[SYSCONFIG3] |= (0x0048);
       break;
     case 2 :
       si4703_registers[SYSCONFIG2] |= (0x0A00);
       si4703_registers[SYSCONFIG3] |= (0x007F);
       break;

     case 3 :
       si4703_registers[SYSCONFIG2] &= (0x00FF);
       si4703_registers[SYSCONFIG3] |= (0x004F);
       break;
  }
  Si4703_UpdateRegisters();
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                fixe la frquence                                       ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_SetChannel(unsigned freq)
{
  //Freq(MHz) = 0.100 * Canal + 87.5MHz
  unsigned newChannel;
  
  newChannel  = freq * 10;
  newChannel -= 8750;
  newChannel /= 10;

  si4703_ReadRegisters();
  si4703_registers[CHANNEL] &= 0xFE00;
  si4703_registers[CHANNEL] |= newChannel;
  si4703_registers[CHANNEL] |= (1<<TUNE);
  si4703_UpdateRegisters();

  while(1) {
    si4703_ReadRegisters();
    if( (si4703_registers[STATUSRSSI] & (1<<STC)) != 0) break;
  }

  si4703_ReadRegisters();
  si4703_registers[CHANNEL] &= ~(1<<TUNE);
  si4703_UpdateRegisters();

  while(1) {
    si4703_ReadRegisters();
    if( (si4703_registers[STATUSRSSI] & (1<<STC)) == 0) break; //Tuning complete!
  }
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                fixe le volume                                          ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_SetVolume(char vol)
{
  Si4703_ReadRegisters();
  if(vol < 0) vol = 0;
  if (vol > 30) vol = 30;
  if (vol < 16)
  {
     si4703_registers[SYSCONFIG3] |= (1<<VOLEXT);
  }
  else
  {
     si4703_registers[SYSCONFIG3] &= 0xFEFF;
     vol -= 15;
  }
  
  si4703_registers[SYSCONFIG2] &= 0xFFF0;
  si4703_registers[SYSCONFIG2] |= vol;
  Si4703_UpdateRegisters();
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                Lecture Donnes RDS                                     ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
char Si4703_RDSRdy(rds *trame)
{
   char flag;
   char BlerA, BlerB, BlerC, BlerD;
   
   Si4703_ReadRegisters();
   if (si4703_registers[STATUSRSSI] & (1<<RDSR))
   {
     BlerA = (si4703_registers[STATUSRSSI] & 0x0600) >> 9;
     BlerB = (si4703_registers[READCHAN]  & 0xC000) >> 14;
     BlerC = (si4703_registers[READCHAN]  & 0x3000) >> 12;
     BlerD = (si4703_registers[READCHAN]  & 0x0300) >> 8;
     
     trame->erreur_1 = BlerD; // Qualit dcodage RDS <3 pour affichage correct du nom de la station
     trame->erreur_2 = BlerB + BlerC + BlerD; // si 0 alors heure correcte
     trame->groupe_A = si4703_registers[RDSA] ;
     trame->groupe_B = si4703_registers[RDSB] ;
     trame->groupe_C = si4703_registers[RDSC] ;
     trame->groupe_D = si4703_registers[RDSD] ;
     trame->valide   = 1;
     flag = 1;
   }
   else
   {
     trame->erreur_1 = 0xFF;
     trame->erreur_2 = 0xFF;
     trame->groupe_A = 0xFFFF;
     trame->groupe_B = 0xFFFF;
     trame->groupe_C = 0xFFFF;
     trame->groupe_D = 0xFFFF;
     trame->valide   = 0;
     flag = 0;
   }
   return(flag);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                RDS Synchronis                                         ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
char Si4703_IsRDS()
{
   char flag;

   Si4703_ReadRegisters();
   if (si4703_registers[STATUSRSSI] & (1<<RDSS))
   {
     flag = 1;
   }
   else
   {
     flag = 0;
   }
   return(flag);
}
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
////                Lecture Donnes RDS                                     ////
////                                                                        ////
////////////////////////////////////////////////////////////////////////////////
////                                                                        ////
void Si4703_ReadRDS(rds *trame)
{
   unsigned short index;
   short          signe;
   int            yy, mo;
   unsigned short kk;
   trame->type_groupe = (trame->groupe_B >> 11);
   trame->pi          = trame->groupe_A;
   trame->pty         = (trame->groupe_B & 0x03E0) >> 5;
   
   if (trame->type_groupe < 3) // type 0A ou 0B
   {
      if (trame->cnt_ps_trame == 0) strcpy(trame->ps,"        ");
      trame->num_ps_trame = (unsigned short) (trame->groupe_B & 0x0003);
      index = trame->num_ps_trame * 2;
      *(trame->ps + index)     = (char)(trame->groupe_D >> 8);
      *(trame->ps + index + 1) = (char)(trame->groupe_D & 0x00FF);
      
      trame->cnt_ps_trame |= (1<<trame->num_ps_trame);
      if (trame->cnt_ps_trame == 0xF)
      {
         trame->ps_ok = 1;
         trame->cnt_ps_trame = 0;
      }
      else trame->ps_ok = 0;
   }
   if (trame->type_groupe == 4) //type 2A
   {
      trame->num_rt_trame = (unsigned short) (trame->groupe_B & 0x000F);
      index = trame->num_rt_trame * 4;
      *(trame->rt + index)     = (char)(trame->groupe_C >> 8);
      *(trame->rt + index + 1) = (char)(trame->groupe_C & 0x00FF);
      *(trame->rt + index + 2) = (char)(trame->groupe_D >> 8);
      *(trame->rt + index + 3) = (char)(trame->groupe_D & 0x00FF);

      trame->cnt_rt_trame |= (1<<trame->num_rt_trame);
      if (trame->cnt_rt_trame == 0xFFFF)
      {
         *(trame->rt + 64) = '\0';
         trame->rt_ok = 1;
         trame->cnt_rt_trame = 0;
      }
      else trame->rt_ok = 0;
   }
   if (trame->type_groupe == 8) //type 4A
   {

      signe         = (trame->groupe_D & 0x0020) >> 5;
      trame->fuseau = (trame->groupe_D & 0x001F) >> 1;
      if (signe == 1) trame->fuseau *= -1;
      trame->mm     = (trame->groupe_D & 0x0FC0) >> 6;
      trame->hh     = (unsigned short)((trame->groupe_C & 0x0001) << 4) + (unsigned short)((trame->groupe_D & 0xF000) >> 12);
      trame->date   = (unsigned long)((trame->groupe_B & 0x0003) << 15) + (unsigned long)((trame->groupe_C) >> 1);
      yy = (unsigned short)floor(((float)trame->date - 15078.2) / 365.25);
      mo = (unsigned short)(floor(((float)trame->date - 14956.1 - floor((float)yy * 365.25)) / 30.6001));
      trame->dd     = (unsigned short)(trame->date - 14956 - floor((float)yy * 365.25) - floor((float)mo * 30.6001));
      if ((mo == 14) || (mo = 15)) kk = 1;
      else kk = 0;
      trame->yy = yy + kk;
      trame->mo = mo - 1 - kk * 12;
      trame->heure_ok = 1;
   }
   else trame->heure_ok = 0;
   
}